'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _array = require('../utils/array');

// This is the method for getting the current viewed page
var getCurrentPage = function getCurrentPage(pages) {
  // To get the current page we find the last page which isn't dismissed
  var index = (0, _array.findLastIndex)(pages, function (item) {
    return !item.meta.dismissed;
  });
  // In order to comply with the default findIndex function in JS we return
  // -1 if there are no pages in the index, otherwise we return the index of
  // the page
  return index >= 0 ? index : -1;
};

// Action to update the page list before navigating to a new page
// Used to store scroll positions for when a user goes back and to remove
// dismissed pages when a user navigates forward
var beforeNavigate = function beforeNavigate(pages) {
  // If there are no pages currently, there is nothing really to do
  if (pages.length <= 0) {
    return pages;
  }
  // We get the index of the current page
  var currentPageIndex = getCurrentPage(pages);
  // And the page itself
  var currentPage = pages[currentPageIndex];
  // And its ID
  var currentPageId = currentPage.id;
  // And tries to get that pages element from the dom
  var currentPageDOM = document.querySelector('div[data-page-id="' + currentPageId + '"]');

  // If it is not being displayed we simply exit
  if (!currentPageDOM) return pages;

  // If it is being displayed we grap all elements with the scroll class
  var scrollElements = currentPageDOM.querySelectorAll('.scroll');
  // And find all their scroll positions
  var scrollPositions = [].map.call(scrollElements, function (elm) {
    return elm.scrollTop;
  });
  // And store them as the current pages scroll positions in its extra
  pages[currentPageIndex].extra.scrollpositions = scrollPositions;
  // And then we return a list with all dismissed pages removed
  return pages.filter(function (p) {
    return !p.meta.dismissed;
  });
};

// After we have navigated to the new page we want to update the pages meta
// information (tite, description etc.)
// we also want to restore scroll positions if there are any stored for the page
var afterNavigate = function afterNavigate(pages) {
  // We start by getting the index of the current page
  var currentPageIndex = getCurrentPage(pages);
  // And the page itself
  var currentPage = pages[currentPageIndex];
  // And it's ID
  var currentPageId = currentPage.id;
  // And its element, if it is in the dom
  var currentPageDOM = document.querySelector('div[data-page-id="' + currentPageId + '"]');

  // If not, there is not much we can do.
  if (!currentPageDOM) return;

  // If it is then we get the cached scroll positions
  var scrollPositions = currentPage.extra.scrollpositions || [];
  // And find all the positions on the page which supports scroll
  var scrollElements = currentPageDOM.querySelectorAll('.scroll');

  // To ensure the page has not changed beyond our expectation we ensure that there
  // are the same amount of scroll elements as when the page was cached
  if (scrollElements.length === scrollPositions.length) {
    // We loop through all our scroll containers
    [].forEach.call(scrollElements, function (elm, i) {
      // And create a reusable function for scrolling to the correct position
      var scrollTo = function scrollTo() {
        // We try to scroll to the correct location
        elm.scrollTop = scrollPositions[i];
        // And returns if it succeded or not (If the element is large enough)
        return elm.scrollTop === scrollPositions[i];
      };
      // We then tries to scroll to the location
      if (!scrollTo()) {
        (function () {
          // If we could not scroll to the correct location, rendering most likely
          // takes more than one pass, so instead we start to listen for DOM
          // manipulations to the container
          var observer = new MutationObserver(function () {
            // And when the DOM is opdated we try again, ind remove our listener
            // if successfull
            if (scrollTo()) observer.disconnect();
          });
          observer.observe(elm, {
            childList: true
          });
          // We also add a timeout, so if the element never will be able to
          // scroll to the correct location, we does free up the reources
          setTimeout(function () {
            scrollTo();
            observer.disconnect();
          }, 1500);
        })();
      }
    });
  }
  // If the current page has meta date to it
  if (currentPage.meta) {
    var meta = currentPage.meta;
    // We get the page tag for the page
    var titleDom = document.querySelector('title');
    // and if it exists
    if (titleDom) {
      // We set it to the pages title
      titleDom.innerHTML = meta.title;
    }
    // We get the description tag for the page
    var descriptionDom = document.querySelector('meta[name="description"]');
    // and if it exists
    if (descriptionDom) {
      // We set it to the pages description
      descriptionDom.innerHTML = meta.description || '';
    }
  }
};

// used when navigation back, so back can close entire modals
var onBack = function onBack(pages, meta) {
  // if the request is to close the model
  if (meta.closeModal) {
    var _ret2 = function () {
      // we get the current page index
      var currentPage = getCurrentPage(pages);
      // and use it to get the modal level of the current page;
      var currentModal = pages[currentPage].meta.modal;
      // and loop through all pages
      return {
        v: pages.map(function (page) {
          // if the current page is the same modal level
          if (page.meta.modal === currentModal) {
            // we set it to dismissed
            page.meta.dismissed = true;
          }
          // and we return the updated list
          return page;
        })
      };
      // if it is not a close modal request
    }();

    if ((typeof _ret2 === 'undefined' ? 'undefined' : _typeof(_ret2)) === "object") return _ret2.v;
  } else {
    var _ret3 = function () {
      // we find the last page in the list which has not been dismissed
      var index = (0, _array.findLastIndex)(pages, function (item) {
        return !item.meta.dismissed;
      });
      // and look if it has a tabKey to it
      var pageKey = pages[index].meta.tabKey;
      // and we dismissed that page
      pages[index].meta.dismissed = true;
      // if the page has a tab key
      if (pageKey) {
        // we find the last inde where this isn't the tab key
        var pageKeyIndex = (0, _array.findLastIndex)(pages, function (item) {
          return item.meta.tabKey !== pageKey && !item.meta.dismissed;
        });
        // if there are any
        if (pageKeyIndex > -1) {
          for (var i = pageKeyIndex + 1; i < pages.length; i++) {
            pages[i].meta.dismissed = true;
          }
        }
      }
      return {
        v: pages
      };
    }();

    if ((typeof _ret3 === 'undefined' ? 'undefined' : _typeof(_ret3)) === "object") return _ret3.v;
  }
};

var mapToHistory = function mapToHistory() {
  var historyState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var reduxState = arguments[1];

  // We only want the content part of the state to be used in history
  reduxState.content = historyState.content;
  return reduxState;
};

var defaultUrl = function defaultUrl(pages) {
  var meta = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (meta.closeModal) {
    return meta.defaultUrl || '/';
  } else {
    return false;
  }
};

exports.default = function (base) {
  return _extends({}, base, {
    getCurrentPage: getCurrentPage,
    beforeNavigate: beforeNavigate,
    onBack: onBack,
    mapToHistory: mapToHistory,
    afterNavigate: afterNavigate,
    defaultUrl: defaultUrl
  });
};